# $Id: LP.pm 543 2008-01-24 11:16:37Z icabrera $
package TAG::LP;

require 5.005;
use strict;
use warnings;
use TAG;
use TAG::Tools;
##use bytes;
use UNIVERSAL qw(isa);

our $map;
our %fset=();
our %features = ();

sub prefix {
    return "\t" x shift;
}

sub lower_or_quote {
    my $string = shift;
    $string = "" unless (defined $string);
    return quote($string) if (defined $map->{lp}{no_normalize});
    $string =~ tr/-/_/;
    return lc($string);
}

sub quote {
    my $string = shift;
    $string = "" unless (defined $string);
    return quote_aux($string) if $string =~ /\s/o;
    return quote_aux($string) if $string =~ /^:+$/o;
    return join('!',map(quote_aux($_),split(/:/,$string)))
}

sub quote_aux {
  my $string = shift;
  $string = "" unless (defined $string);
  return $string if ($string =~ /^[a-zA-Z_]\w*$/o && $string !~ /^[_A-Z]/ );
  return $string if ($string =~ /^\d+$/);
  #    $string =~ s/\'/\'\'/og;
  $string =~ s/\'(?!\')/\'\'/og;
  return "\'$string\'";
}

sub var {
    my $string = shift;
    $string =~ s/\W/_/og;
    return "_$string";
}

sub single_or_list {
    my $o = shift ;
    if (@{$o} == 1) {
	return $o->[0];
    } else {
	return &list($o,@_);
    }
}

sub list {
    my $o = shift;
    my $format = shift;
    return $format->{'open'} . join( $format->{'sep'}, @$o ) . $format->{'close'}; 
}

sub header_reader {
    my $header = shift;
    open(HEADER,"< $header") || die "can't open header file $header";
    while(<HEADER>) {
	if (/^:-finite_set\(\s*(\S+?)\s*,/) {
#	    print STDERR "SETTING FSET $1\n";
	    $fset{$1}=1;
	}
	if (/^:-features\(\s*(\S+?)\s*,/) {
#	    print STDERR "SETTING FEATURE $1\n";
	    $features{$1}=1;
	}
    }
    close(HEADER);
}

sub TAG::Document::LP {
    my $self = shift;
    $map = shift || {};
    %fset=();
    my @includes = ();
    # Reading header file if specified for finite set declarations
    if (defined $map->{'lp'}{'header'}){
	my $header = $map->{'lp'}{'header'};
	header_reader($header);
	push(@includes,$header);
    }
    my @lexkeys = sort (keys %{$self->{'lexicon'}});
    my @lemma = ();
    foreach my $lemmakey (sort (keys %{$self->{'lemma'}})) {
	my $tmp = $self->{'lemma'}{$lemmakey};
	foreach my $cat (sort (keys %$tmp )){
	    push @lemma, $tmp->{$cat};
	}
    }
    my @trees = ();
    my $done = {};
    foreach my $family (sort (keys %{$self->{'families'}})) {
	push(@trees,$self->{'families'}{$family}->LP($self,$done));
    }
    my $s= ( join("\n", map( $_->LP(), @{ $self->{'desc'} }))
	     . "\n"
	     . join("\n",map( ":-include('$_').", @includes)) . "\n"
	     . join("\n",
		    map( ($self->{'lexicon'}{$_})->LP() , @lexkeys  ))
	     . "\n\n"
	     . join("\n\n",
		    map( $_->LP() , @lemma  ))
	     . "\n\n"
	     . join("\n\n",@trees) 
	     . "\n\n"
	     );
##    $s =~ tr/\0-\x{ff}//UC;
##    $s = UTF8toISO($s);
    return $s;
}

sub TAG::Family::LP {
    my $self = shift;
    my $doc = shift;
    my $done = shift;
    my @keys = sort @{$self->{'trees'}};
    my @trees = ();
    push(@trees,"%% Family $self->{'name'}: {" . join(',',@keys) . "}" );
    foreach my $tree (@keys) {
	if ($done->{'trees'}{$tree}) {
	    push(@trees,"%% Tree $tree already emitted for family $done->{'trees'}{$tree}");
	} else {
	    push(@trees,$doc->{'trees'}{$tree}->LP());
	    $done->{'trees'}{$tree} = $self->{'name'};
	}
    }
    return @trees;
}

sub TAG::Desc::LP {
    my $self = shift;
    return "%% $self->{'text'}";
}

sub TAG::Tree::LP {
    my $self = shift;
    my @args = ();
    my $type = $self->{'type'};
    my @family = map TAG::LP::quote($_), @{ $self->{'family'} };
    @family = ($self->{ht}->LP('ht'))
      if (defined $map->{'lp'}{'hypertag'} && defined $self->{ht});
    $type = ($type eq 'initial' ? 'tree' : 'auxtree' );
    my $name = &TAG::LP::quote($self->{'name'});
    push(@args, 
	 "name => " . $name,
	 "family => " . TAG::LP::single_or_list(\@family, { open => '[', close => ']', sep=>',' }),
	 "tree => $type " . $self->{'root'}->LP(3,@{$self->{reducers}})
	 );
    my $output = "tag_tree" . &TAG::LP::list(\@args,
					     { open=> "{\t",
					       close=> "\n\t}",
					       sep=> ",\n\t\t"
					     }) . ".";
    return $output;
}

sub TAG::Optional::LP {
  my $self = shift;
  my $output = $self->{node}->LP(@_);
  my $n = $self->{n};
  my $mark = "?";
  $mark = '@*' if ($n eq '*');
  return @{$self->{node}{guards}} ? "$output" : "($output)$mark";
}


sub guard_wrapper {
  my $self = shift;
  my $display = shift;
  if (@{$self->{guards}}) {
    my %guards = map($_->LP,@{$self->{guards}});
    my $plus = $guards{'+'} || 'true';
    my $minus = $guards{'-'} || 'true';
    $display = "guard{ goal => $display, plus => ( $plus ), minus => ( $minus ) }";
  }
  return $display;
}

sub TAG::Interleave::LP {
  my $self = shift;
  my $depth = shift;
  my @child = map($_->LP($depth+1),@{$self->{child}});
  my $display = &TAG::LP::list( \@child,
			       { open => "(\n" . TAG::LP::prefix($depth+1), 
				 close => "\n" . TAG::LP::prefix($depth) . ")", 
				 sep => "\n" . TAG::LP::prefix($depth) . "##      " });
  return guard_wrapper($self,$display);
}

sub TAG::Sequence::LP {
  my $self = shift;
  my $depth = shift;
  my @child = map($_->LP($depth+1),@{$self->{child}});
  my $display = &TAG::LP::list( \@child,
				{ open => "(\n" . TAG::LP::prefix($depth+1), 
				  close => "\n" . TAG::LP::prefix($depth) . ")", 
				  sep => "\n" . TAG::LP::prefix($depth) . ",       " });
  return guard_wrapper($self,$display);
}

sub TAG::Alternative::LP {
  my $self = shift;
  my $depth = shift;
  my @child = map($_->LP($depth+1),@{$self->{child}});
  my $display = &TAG::LP::list( \@child,
				{ open => "(\n" . TAG::LP::prefix($depth+1), 
				  close => "\n" . TAG::LP::prefix($depth) . ")", 
				  sep => "\n" . TAG::LP::prefix($depth) . ";       " });
  return guard_wrapper($self,$display);
}


sub TAG::Node::LP {
    my $self = shift;
    my $depth = shift;
    my @reducers = @_;
    my $cat = TAG::LP::lower_or_quote($self->{'cat'});
    my $type = $self->{'type'};
    my $lex = $self->{'lex'};
    my $adj = $self->{'adj'};
    my $id = $self->{'id'};
    my $top = $self->{'top'};
    my $bot = $self->{'bot'};
    my $token = $self->{token};
    my @child = @{$self->{'child'}};
    my $child='';
    my @args = ();
    push(@args, "id= " . &TAG::LP::quote($id) ) if ($id);
    push(@args, "top= " . $top->LP($cat)) if ($top);
    push(@args, "bot= " . $bot->LP($cat)) if ($bot);
    push(@args, "token= ", $bot->LP::quote($token)) if ($token);
    my $args = "";
    if (@args) {
	$args = &TAG::LP::list( \@args,
				{ open => '', 
				  close => "\n" . TAG::LP::prefix($depth) . "at ", 
				  sep => "\n" . TAG::LP::prefix($depth) . "and " }
				);
    }
    @child = map $_->LP($depth+1), @child;
    ## We have a pb if Node has no child but reducers
    ## that case should not happen !
    push(@child,'{'.join(', ',map($_->LP($depth+1),@reducers)).'}') if (@reducers);
    $child= &TAG::LP::list( \@child,
			    { open => "(\n" . TAG::LP::prefix($depth+1), 
			      close => "\n" . TAG::LP::prefix($depth) . ")", 
			      sep => ",\n" . TAG::LP::prefix($depth+1) }
			    ) if (@child);
    my $label = $cat;
    if ($type eq 'lex') {
      if (!defined $lex) {  
	$label = '[_]';
      } elsif ($lex eq '') {
	$label = '[]';
      } else {
	$label = '[' . TAG::LP::quote($lex) . ']' ;
      }
    }
    $label = "<> $label" if ($type eq 'anchor');
    $label = "<=> $label" if ($type eq 'coanchor');
    $label = "* $label"  if ($type eq 'foot');
    my $mask = '';
    if (defined $adj && ($type eq 'anchor' || $type eq 'std' ||$type eq 'coanchor')) {
      $mask = '- ' if ($adj eq 'no');
      $mask = '++ ' if ($adj eq 'strict');
    }
    my $display = "$args$mask$label$child";
    return guard_wrapper($self,$display);
}

sub TAG::Reducer::LP {
  my $self = shift;
  return "atom(".$self->{value}[0]->LP.")";
}

sub TAG::Not::LP {
  my $self = shift;
  my $kind = shift;
  my @values = map $_->LP($kind,'neg'), @{$self->{'value'}};
  $kind = TAG::LP::lower_or_quote($kind);
  return "$kind\[~ [".join(',',@values)."]\]";
}

sub TAG::VAlt::LP {
  my $self = shift;
  my $kind = shift;
  my $neg = shift || '';
  my @values = map $_->LP($kind), @{$self->{'value'}};
  $kind = TAG::LP::lower_or_quote($kind);
  my $res = join(',',@values);
  $res = "$kind\[$res\]" unless ($neg);
  return $res;
}


sub TAG::Var::LP {
  my $self = shift;
  my $kind = shift;
  my $name = TAG::LP::var($self->{name});
  my @values = map $_->LP($kind), @{$self->{'value'}};
  $kind = TAG::LP::lower_or_quote($kind);
  my $output = $name;
  return $output unless @values;
  $output .= ' :: ';
  if (@values > 1 
      ## following line not necessary, except to mark
      ## single values from finite set domains
      ## || (defined $fset{$kind} && !isa($self->{value}[0],"TAG::Feature"))
     ) {
    $output .= "$kind\[".join(',',@values)."\]";
  } else {
    $output .= $values[0];
  }
  return $output;
}

sub TAG::Or::LP {
  my $self =shift;
  my @guards = @{$self->{'guards'}};
  return 'fail' unless (@guards);
  return $guards[0]->LP if (@guards == 1);
  return '( '.join( ' ; ', map($_->LP,@guards)).' )';
}

sub TAG::And::LP {
  my $self =shift;
  my @guards = @{$self->{'guards'}};
  return 'fail' unless (@guards);
  return $guards[0]->LP if (@guards == 1);
  return '( '.join( ' , ', map($_->LP,@guards)).' )';
}


sub TAG::NGuards::LP {
  my $self = shift;
  my @guards = @{$self->{'guards'}};
  return @guards ? ($self->{rel} => join(',',map($_->LP,@guards))) : ();
}

sub TAG::NGuard::LP {
  my $self = shift;
  my $kind = $self->{type} || "none";
  $kind = $map->{fs_redirect}{$kind} if (defined $map->{fs_redirect}{$kind});
  return join( ' = ', map($_->LP($kind),@{$self->{'values'}}) );
}

sub TAG::NXGuard::LP {
  my $self = shift;
  my $kind = $self->{type} || "none";
  $kind = $map->{fs_redirect}{$kind} if (defined $map->{fs_redirect}{$kind});
  my $ret = join( ' = ', map($_->LP($kind),@{$self->{'values'}}) );
  return "($ret xor true)";
}

sub TAG::Lemma::LP {
    my $self = shift;
    my $name = TAG::LP::quote($self->{'name'});
    my $cat = TAG::LP::lower_or_quote($self->{'cat'});
    my @anchors = map $_->LP(), @{ $self->{'anchors'} };
    my $anchors = &TAG::LP::single_or_list(\@anchors,
					   { open => "[ ",
					     close => "\n\t]",
					     sep => ",\n\t  "
					     });
    return "tag_lemma($name,$cat,\n\t$anchors\n\t).";
}

sub TAG::Lexicon::LP {
    my $self = shift;
    my $lex = TAG::LP::quote($self->{'lex'});
    return map( $_->LP($lex), @{$self->{'lemma'}} );
}

sub TAG::LemmaRef::LP {
    my ($self,$lex) = @_;
    my $name = TAG::LP::quote($self->{'name'});
    my $cat = TAG::LP::lower_or_quote($self->{'cat'});
    my $feature = $self->{'feature'};
    if ($feature) {
	$feature = $feature->LP($cat);
    } else {
	$feature = '_';
    }
    return "tag_lexicon($lex," . $name . ", $cat, $feature).";
}

sub TAG::Feature::LP {
    my $self = shift;
    my $kind = $self->{'type'};
    $kind ||= shift;
    my @features = map $self->{'f'}{$_}->LP, sort (keys %{$self->{'f'}});
    if (@features || defined $features{$kind}) {
	return "$kind"."{" . join(', ',@features) . "}";
    } else {
	return "$kind";
    }
}

sub TAG::Comp::LP {
  my $self = shift;
  my $name = $self->{'name'};
  my $kind = (defined $map->{fs_redirect}{$name}) ? $map->{fs_redirect}{$name} : $name;
  my @values = map $_->LP($kind), @{$self->{'value'}};
  $kind = TAG::LP::lower_or_quote($kind);
  $name = TAG::LP::lower_or_quote($name);
  return unless (@values);
  my $output = "$name => ";
  if (@values > 1 
      ## following line not necessary, except to mark
      ## single values from finite set domains
      ## || (defined $fset{$kind} && !isa($self->{value}[0],"TAG::Feature"))
     ) {
    $output .= "$kind\[".join(',',@values)."\]";
  } else {
    $output .= $values[0];
  } 
  return $output;
}

sub TAG::Narg::LP {
    my $self = shift;
    my $cat = shift;
    my $name = $self->{'name'};
    my $kind = (defined $map->{fs_redirect}{$cat}) ? $map->{fs_redirect}{$cat} : $cat;
    my @values = map $_->LP($kind), @{$self->{'value'}};
    $kind = TAG::LP::lower_or_quote($kind);
    $name = TAG::LP::lower_or_quote($name);
    return unless (@values);
    my $output = "";
    if (@values > 1 
	## following line not necessary, except to mark
	## single values from finite set domains
	## || (defined $fset{$kind} && !isa($self->{value}[0],"TAG::Feature"))
       ) {
      $output .= "$kind\[".join(',',@values)."\]";
    } else {
      $output .= $values[0];
    } 
    return $output;
  }


sub TAG::Equation::LP {
  my $self = shift;
  my $type = &TAG::LP::quote($self->{'type'});
  my $node_id = $self->{'node_id'};
  my $fs = $self->{'feature'};
  my ($node_cat) = ($node_id =~ /^(\w+)_/);
  $node_cat = TAG::LP::lower_or_quote($node_cat);
  $node_id = TAG::LP::quote($node_id);
  return "$type = " . $fs->LP($node_cat) . " at $node_id";
}

sub TAG::Anchor::LP {
    my $self = shift;
    my @args = ();
    my @families = map &TAG::LP::quote($_), @{ $self->{'family'} };
    push(@args,
	 "name => " . &TAG::LP::single_or_list(\@families,
					       { open => '[ ',
						 close => ' ]',
						 sep => ', '
						 }));
    my @coanchors = map $_->LP(), @{ $self->{'coanchors'} };
    if (@coanchors) {
	push(@args,
	     "coanchors => " . &TAG::LP::list(\@coanchors,
					      { open => '[ ',
						close => ' ]',
						sep => ', '
						}));
    }
    my @equations = map $_->LP(), @{ $self->{'equations'} };
    if (@equations) {
	push(@args,
	     "equations => " . &TAG::LP::list(\@equations,
					      { open => '[ ',
						close => ' ]',
						sep => ",\n\t\t\t\t       "
						}));
    }
    return "tag_anchor" . &TAG::LP::list( \@args,
					  { open => "{\t",
					    close => ' }',
					    sep => ",\n\t\t\t"
					    }
					 );
}

sub TAG::Coanchor::LP {
    my $self = shift;
    my $node_id = TAG::LP::quote($self->{'node_id'});
    my @lex = map $_->LP(), @{$self->{'lex'} };
    my $lex = &TAG::LP::single_or_list(\@lex,
				       { open => '(',
					 close => ')',
					 sep => ';' 
					 });
    return "$node_id=$lex";
}

sub TAG::Val::LP {
    my $self = shift;
    return TAG::LP::quote( $self->{'text'} );
}

sub TAG::Lex::LP {
    my $self = shift;
    return TAG::LP::quote( $self->{'text'} );
}

sub TAG::Plus::LP {
    my $self = shift;
    return "(+)" ;
}

sub TAG::Minus::LP {
    my $self = shift;
    return "(-)" ;
}

1;

__END__

=head1 NAME

TAG::LP - output content of a Tree Adjoining Grammar (TAG) as a string
in Logic Programming format.

=head1 SYNOPSIS

 use TAG::RCG;
 
 # using LP method on a TAG::Document 
 print $tag_object->LP();

=head1 DESCRIPTION

Calling `C<LP>' on an TAG object returns it as string in Logic
Programming format. It may then be compiled using system 'C<DyALog>'.

=head1 OPTIONS

    none.

=over 4

=back

=head1 EXAMPLES

Here is an example of parsing a TAG in XML format and emitting it as a LP:

    use TAG::XML::Parser;
    use TAG::LP;

    my $parser = TAG::XML::Parser->new ();
    my $doc = $parser->parsefile( $grammarfile );

    print $doc->LP();

=head1 COPYRIGHT AND LICENSE

Copyright (C) 2000-2007, INRIA.

This program is free software; you can redistribute it and/or modify it under
the same terms as Perl itself.

=head1 AUTHOR

Eric de la Clergerie <Eric.De_La_Clergerie@inria.fr>

=head1 SEE ALSO

TAG (3)

DyALog <http://alpage.inria.fr/~clerger>

=cut
